/****************************************************************************
 *																			*
 * File    : loadrc.c														*
 *																			*
 * Purpose : Sample add-in DLL. 											*
 *																			*
 * History : Reason															*	 
 * 					To load an RC file into the editor						*
 *			 Created 														*
 *					4th Oct 2005 (JohnF)									*
 *																			*
 ****************************************************************************/

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include <addin.h>
#include <stdio.h>
#include <stdlib.h>
#include "loadrc.h"

// Private command ID - any number will do.
#define CMD_OPENRC 1

// Locals.
static HANDLE g_hmod = NULL;
static HWND g_hwndMain = NULL;
static HWND g_hwndPrj  = NULL;
static HWND g_hwndRC[3];			// number of allowed RC files == 3
static int  g_numRCopened = 0;		// count them

/****************************************************************************
 *																			*
 * Function: DllMain														*
 *																			*
 * Purpose : DLL entry and exit procedure.									*
 *																			*
 * History : Date	   Reason												*
 *			 04-01-10  Created												*
 *																			*
 ****************************************************************************/
BOOL WINAPI DllMain(HANDLE hDLL, DWORD dwReason, LPVOID lpReserved)
{
	switch (dwReason)
	{
		case DLL_PROCESS_ATTACH:
			g_hmod = hDLL;
			return TRUE;

		case DLL_PROCESS_DETACH:
			return TRUE;

		default:
			return TRUE;
	}
}

int __cdecl compare(const void * x, const void * y)
{
	// sort them so that a '0' will be last
	int a = *(int*)x;
	int b = *(int*)y;
	if(a < b) 
		return 1;
	if(a > b) 
		return -1;
	return 0;
} 

/****************************************************************************
 *																			*
 * Function: AddInMain														*
 *																			*
 * Purpose : Add-in entry and exit procedure.								*
 *																			*
 * History : Date	   Reason												*
 *			 04-01-10  Created												*
 *																			*
 ****************************************************************************/
ADDINAPI BOOL WINAPI AddInMain(HWND hwnd, ADDIN_EVENT eEvent)
{
	static BOOL prj_destroy = FALSE;
	switch (eEvent)
	{
		case AIE_DOC_DESTROY:
		{
			// if AIE_PRJ_DESTROY is destorying the windows don't do anything
			if(prj_destroy == TRUE)
				return TRUE;

			BOOL wasWnd = FALSE;
			// check to see if the closed windows was an RC ifle
			for(int i = 0; i < 3; i++)
			{
				if(hwnd == g_hwndRC[i])
				{
					g_hwndRC[i] = 0;
					g_numRCopened--;
					wasWnd = TRUE;
				}
			}
			if(wasWnd)
			{
				// sort the array g_hwndRC
				qsort(g_hwndRC, 3, sizeof(HWND), compare); 
			}

			return TRUE;
		}

		case AIE_APP_CREATE:
		{
			prj_destroy = FALSE;

			ADDIN_ADD_COMMAND AddCmd;

			/* Save handle of main IDE window */
			g_hwndMain = hwnd;

			/* Add command button to IDE toolbar */
			AddCmd.cbSize  = sizeof(AddCmd);
			AddCmd.pszText = "Open resource file in Editor.";  // Tooltip text.
			AddCmd.hIcon   = LoadImage(g_hmod, MAKEINTRESOURCE(IDR_ICON1), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR|LR_SHARED);
			AddCmd.id      = CMD_OPENRC;
			AddCmd.idMenu  = AIM_MENU_NOTHING;
			return AddIn_AddCommand(hwnd, &AddCmd);
		}

		case AIE_PRJ_CREATE:
			/* Save handle of Project window */
			g_hwndPrj = hwnd;
			return TRUE;

		case AIE_PRJ_DESTROY:
			/* The IDE does not like having a window open that it did
			   not load, close the RC source windows before app termintaes */
			prj_destroy = TRUE;
			for(int i = 0; i < 3; i++)
			{
				if (g_hwndRC[i])
					AddIn_CloseDocument(g_hwndRC[i]);
				g_hwndRC[i] = 0;
			}
			g_numRCopened = 0;
			return TRUE;

		case AIE_APP_DESTROY:
			/* Remove button from IDE toolbar */
			return AddIn_RemoveCommand(hwnd, CMD_OPENRC);

		default:
			return TRUE;
	}
}

BOOL CALLBACK RCFilesCallback(LPCTSTR pszFilename, LPVOID pvData)
{
	char * p = pvData;
	if(_stristr(pszFilename, ".rc"))
	{
		strcpy(p, pszFilename);
		return FALSE; // found it, stop enum
	}
	return TRUE;
}

char * GetRCName(void)
{
	static char szRCEnum[260];
	szRCEnum[0] = '\0';
	ADDIN_ENUM_PROJECT_FILES ePrjFiles;
	ePrjFiles.cbSize 		= sizeof(ADDIN_ENUM_PROJECT_FILES);
	ePrjFiles.pfnCallback 	= RCFilesCallback;
	ePrjFiles.fuFlags 		= EPFF_DEPENDENT_FILES;
	ePrjFiles.pvData 		= szRCEnum;
	AddIn_EnumProjectFiles(g_hwndPrj, &ePrjFiles);
	return szRCEnum;
}

void DoOpenRC(void)
{
	// Get the RC path and name
	char * szRCPath = GetRCName();
	if (!szRCPath[0])
	{
		MessageBox(g_hwndMain, "Could not find the RC File!", "LoadRC Error!", MB_OK);
		return;
	}
	AddIn_ClearOutput(g_hwndMain);
	AddIn_WriteOutput(g_hwndMain, szRCPath);
	if(g_numRCopened < 3)
	{
		g_hwndRC[g_numRCopened++] = AddIn_OpenDocument(g_hwndMain, AID_SOURCE, szRCPath);
	}
	else
	{
		MessageBox(g_hwndMain, "Too many opened RC files!", "LoadRC Error!", MB_OK);
	}
}

/****************************************************************************
 *																			*
 * Function: AddInCommand													*
 *																			*
 * Purpose : Add-in command handler.										*
 *																			*
 * History : Date	   Reason												*
 *			 04-01-10  Created												*
 *																			*
 ****************************************************************************/
ADDINAPI void WINAPI AddInCommand(int idCmd)
{
	switch (idCmd)
	{
		case CMD_OPENRC:
			DoOpenRC();
			break;
	}
}

